/**********************************************************************************

Copyright (c) 1992 by Hewlett-Packard Company, Palo Alto, California

                        All Rights Reserved

Permission to use, copy, modify, and distribute this software and its
documentation for any purpose and without fee is hereby granted,
provided that the above copyright notice appear in all copies and that
both that copyright notice and this permission notice appear in
supporting documentation, and that the name of Hewlett-Packard not be
used in advertising or publicity pertaining to distribution of the
software without specific, written prior permission.

HEWLETT-PACKARD DISCLAIMS ALL WARRANTIES WITH REGARD TO THIS SOFTWARE, INCLUDING
ALL IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS, IN NO EVENT SHALL
HEWLETT-PACKARD BE LIABLE FOR ANY SPECIAL, INDIRECT OR CONSEQUENTIAL DAMAGES OR
ANY DAMAGES WHATSOEVER RESULTING FROM LOSS OF USE, DATA OR PROFITS,
WHETHER IN AN ACTION OF CONTRACT, NEGLIGENCE OR OTHER TORTIOUS ACTION,
ARISING OUT OF OR IN CONNECTION WITH THE USE OR PERFORMANCE OF THIS
SOFTWARE.

**********************************************************************************/
/******* Modified by Volker Senft, Logitech Inc., Fremont, California ************/

#include <stdio.h>
#include <fcntl.h>
#include <termio.h>
#include <termios.h>
#include <unistd.h>
#include <ctype.h>
#include <stdlib.h>
#include <string.h>
#include <signal.h>
#include <poll.h>
#include <pwd.h>

#include <sys/types.h>
#include <sys/time.h>
#include <sys/modem.h>
#include <sys/termiox.h>

#include "XI.h"
#include "x_serialdrv.h"

#ifndef TRUE
#define TRUE	(1==1)
#endif

#ifndef FALSE
#define FALSE	(1==0)
#endif

#define MagellanBufferSize	256
static char MagellanBuffer[ MagellanBufferSize ];
static int  MagellanIndex = 0;

#define MagellanNibble(Value)		(Value&0x0F)

static char MagellanInitString[] = "\rz\rz\r";		/* switch in 3D mode */
static char MagellanShortBeep[] = "bH\r";		/* a short beep */
static char MagellanMode[] = "m3\r";			/* translation and rotation data ON */
static char MagellanPeriod[] = "pAA\r";			/* transmit every 60 ms data */
static char MagellanNullRadius[] = "nH\r";		/* null radius to value 8 */
static char MagellanZero[] = "z\r";			/* detect zero position */
static char MagellanSensitivity[] = "q00\r";		/* no extra sensitivity */
static char MagellanVersion[] = "vQ\r";			/* get version string */

static int MagellanKeyboard = 0;


/*--------------- Completely private functions ---------------------------*/ 
static void MagellanWait( WaitTime )
clock_t WaitTime;
{
clock_t start,stop;

 if ( WaitTime >= 1000 )
  {
   sleep( 1 );
   MagellanWait( WaitTime - 1000 );
  }
  else
  {
   start = clock();
   do
    {
     stop = clock();
    }while( labs(stop-start) < WaitTime*CLOCKS_PER_SEC/1000 );
  };
}

static int MagellanReadString( MagellanHandle, MagellanString ) 
int MagellanHandle; 
char *MagellanString; 
{ 
char *cptr;
int result;

 do
  {
   cptr = strchr( MagellanBuffer, '\r' );		/* check if end of packet */
   if ( cptr == NULL )
    {
     result = read( MagellanHandle, &MagellanBuffer[ MagellanIndex ], /* read from the serial port */
                              sizeof(MagellanBuffer)-MagellanIndex );
     if ( result == 0 ) 
      return FALSE;    
     MagellanIndex += result;               
     MagellanBuffer[ MagellanIndex ] = '\000';
    }
   else
    {
     *cptr = '\000';
     strcpy( MagellanString, MagellanBuffer );
     strcpy( MagellanBuffer, ++cptr );
     MagellanIndex = strlen( MagellanBuffer );

     return TRUE;
    };
  } while( TRUE );
} 
 
 
static int MagellanWriteString( MagellanHandle, MagellanString ) 
int MagellanHandle; 
char * MagellanString; 
{
 if ( MagellanHandle == NULL )
  return FALSE;
 else
  write( MagellanHandle, MagellanString, strlen(MagellanString) );  
 return TRUE;
} 
 
/*----------- End of Completely private functions -------------------------*/ 
 
 
 
/*--------- These functions are called from outside this file -------------*/ 
 
static int magellanconfigure (MagellanHeader) 
HPInputDeviceHeader *MagellanHeader; 
{ 
int result, length; 
struct termio MagellanTermio; 
mflag Flags; 
char MagellanString[MagellanBufferSize]; 
 
 MagellanHeader->file_ds = open (MagellanHeader->path, O_RDWR | O_NDELAY ); 
 if (MagellanHeader->file_ds < 0)  
  return INIT_FAILURE;				/* Can't open serial device */ 
 
 Flags = MRTS | MDTR | MDRS; 			/* enable RTS and DTR hardware handshake */
 ioctl (MagellanHeader->file_ds, MCSETA, &Flags ); 
 
 /* configure the serial port for the Magellan 3D Controller */ 
 ioctl (MagellanHeader->file_ds, TCGETA, &MagellanTermio); 
 MagellanTermio.c_lflag = 0; 
 MagellanTermio.c_iflag = IGNBRK | IGNPAR | IXON | IXOFF; 
 MagellanTermio.c_oflag = 0;         	
 MagellanTermio.c_cflag =  B9600 | CREAD | CS8 | CSTOPB | CLOCAL;
 MagellanTermio.c_cc[VEOL] = '\r';  			 
 MagellanTermio.c_cc[VERASE] = CNUL;
 MagellanTermio.c_cc[VKILL] = CNUL;
 MagellanTermio.c_cc[VMIN] = 0; 
 MagellanTermio.c_cc[VTIME] = 0; 
 ioctl (MagellanHeader->file_ds, TCSETA, &MagellanTermio); 

 MagellanHeader->x_name = "MAGELLAN"; 
 MagellanHeader->resolution = 0;          		/* resolution in counts/cm      */ 
 MagellanHeader->max_x = 0;              		/* maximum x value in counts    */ 
 MagellanHeader->max_y = 0;                       	/* maximum y value in counts    */ 
 MagellanHeader->flags = DATA_IS_16_BITS; 		/* device characteristics       */ 
 MagellanHeader->ax_num = 6;              		/* number of axes               */ 
 MagellanHeader->num_buttons = 9;         		/* number of buttons            */ 
 
 MagellanWait( 2000 );					/* wait for the Magellan power up */ 
 
 ioctl (MagellanHeader->file_ds, MCGETA, &Flags ); 	/* Magellan must be ready for receive */
 if ( (Flags & MCTS) != MCTS )
  return INIT_FAILURE;	
 else
  {
   memset( MagellanBuffer, 0x00, sizeof(MagellanBuffer) );
   MagellanIndex = 0;

   MagellanWriteString( MagellanHeader->file_ds, MagellanInitString );
   MagellanWait( 100 );

   MagellanWriteString( MagellanHeader->file_ds, MagellanVersion );
   MagellanWait( 100 );

   MagellanWriteString( MagellanHeader->file_ds, MagellanSensitivity );
   MagellanWait( 100 );

   MagellanWriteString( MagellanHeader->file_ds, MagellanZero );
   MagellanWait( 100 );

   MagellanWriteString( MagellanHeader->file_ds, MagellanPeriod );
   MagellanWait( 100 );

   MagellanWriteString( MagellanHeader->file_ds, MagellanMode );
   MagellanWait( 100 );

   MagellanWriteString( MagellanHeader->file_ds, MagellanNullRadius );
   MagellanWait( 100 );

   MagellanWriteString( MagellanHeader->file_ds, MagellanShortBeep );
  };
 
 MagellanWait( 1000 );					/* wait for request from the Magellan */

 result = INIT_FAILURE;
 while ( MagellanReadString( MagellanHeader->file_ds, MagellanString ) )
  {
   if ( (strstr( MagellanString, "SPACE MOUSE" ) != NULL) || 
        (strstr( MagellanString, "MAGELLAN" ) != NULL) )
    {
     result = INIT_SUCCESS;    
    };
  };
 
 return result;   
} 
 
/************************************************************************** 
 * 
 * This routine is called by the X server to read from the magellan. 
 * 
 */ 
 
static int magellanread ( MagellanHandle, MagellanEvent, MagellanEventType, XPending ) 
int MagellanHandle; 
unsigned char *MagellanEvent; 
unsigned char *MagellanEventType; 
int *XPending; 
{
char MagellanString[ MagellanBufferSize ];
int MagellanButton, loop, mask;
int MagellanData;
 
 do 
  {
   if ( MagellanReadString( MagellanHandle, MagellanString ) == FALSE )
    {
     return READ_FAILURE;		/* no packets available */
    }
   else
    {
     switch( MagellanString[ 0 ] )
      {
       case 'd' :
        for ( loop=0; loop<6; ++loop )
         {
          MagellanData = MagellanNibble( MagellanString[4*loop+1] ) * 4096 +
                         MagellanNibble( MagellanString[4*loop+2] ) * 256 +
                         MagellanNibble( MagellanString[4*loop+3] ) * 16 +
                         MagellanNibble( MagellanString[4*loop+4] ) - 32768;   
          MagellanEvent[2*loop+0] =  MagellanData & 0x00FF;
          MagellanEvent[2*loop+1] = (MagellanData >> 8) & 0x00FF;
         };
        MagellanEvent[ 12 ] = 60; MagellanEvent[ 13 ] = 0;
        *MagellanEventType = MOTION_DATA; 
        *XPending = 14; 
        return READ_SUCCESS; 
          
       case 'k' :
        MagellanButton = MagellanNibble( MagellanString[1] ) * 1 +
                         MagellanNibble( MagellanString[2] ) * 16 +
                         MagellanNibble( MagellanString[3] ) * 256;  
        for ( loop=0; loop<9; ++loop )
         {
          mask = 1 << loop;
          if ( (MagellanButton ^ MagellanKeyboard) & mask )
           {
            MagellanEvent[ 0 ] = 2*loop + ((MagellanButton & mask) ? 0 : 1);
            MagellanEvent[ 1 ] = 0x00;  
            *MagellanEventType = BUTTON_DATA;
            *XPending = 2;
            MagellanKeyboard = MagellanButton;
            return READ_SUCCESS;
           };
         };
        break;
        
      };
    };
  } while ( TRUE );
    
 return READ_FAILURE; 
} 
 
/************************************************************************** 
 * 
 * This routine is called by the X server to writeh request. 
 * 
 */ 
 
static int magellanwrite (MagellanHandle, XRequest, MagellanEvent) 
int MagellanHandle, XRequest; 
char *MagellanEvent; 
{ 
 switch (XRequest) 
  { 
   case _XSetDeviceValuators: 
   case _XChangeDeviceControl: 
   case _XSetDeviceMode:	 
    return WRITE_FAILURE; 
         
   case _XChangeFeedbackControl: 
    MagellanWriteString( MagellanHandle, MagellanShortBeep ); 
    return WRITE_SUCCESS; 
 
   default:			 
    return WRITE_FAILURE; 
  }; 
 return WRITE_SUCCESS; 
} 
 
/***************************************************************/ 
magellanclose (MagellanHandle) 
int MagellanHandle; 
{ 
 close(MagellanHandle); 
 return CLOSE_SUCCESS; 
} 
 
/************************************************************************** 
 * 
 * This routine is called by the X server to initialize the driver. 
 * 
 */ 
 
int magellan_Init(serialproc) 
SerialProcs *serialproc; 
{ 
 serialproc->configure = magellanconfigure; 
 serialproc->read = magellanread; 
 serialproc->write = magellanwrite; 
 serialproc->close = magellanclose; 
 return INIT_SUCCESS; 
} 


